
from nonebot.log import logger

from nonebot_dcqq_relay_plugin.config import plugin_config
from nonebot_dcqq_relay_plugin.Adapters.QQ import QQ, formatImg, formatAT
from nonebot_dcqq_relay_plugin.Database.db import DB, DiscordModule
from nonebot_dcqq_relay_plugin.Core.constants import messageEvent, bot_manager, noticeEvent
from nonebot_dcqq_relay_plugin.Core.global_functions import apngToGif, lottieToGif

from nonebot.adapters.discord import Bot as DiscordBot#, MessageSegment as DiscordMessageSegment, Message as DiscordMessage
from nonebot.adapters.onebot.v11 import Message as OneBotMessage, MessageSegment as OneBotMessageSegment
from nonebot.adapters.discord.event import MessageCreateEvent as DiscordMessageCreateEvent, MessageDeleteEvent as DiscordMessageDeleteEvent

#====================================================================================================

@messageEvent.handle()
async def handle_discord_message(bot: DiscordBot, event: DiscordMessageCreateEvent):
    if not bot_manager.OneBotObj or not isinstance(event, DiscordMessageCreateEvent) or event.channel_id != plugin_config.discord_channel or event.guild_id != plugin_config.discord_guild:
        return;

    await DiscordModule.Create(str(event.id))

    # 此检测是为了防止转发机器人抽风
    if event.webhook_id == bot_manager.webhook_id:
        logger.debug(f"检测Webhook [Webhookid: {event.webhook_id}]");
        return;

    QQFunc = QQ(userNick=event.member.nick, globalName=event.author.global_name ,userName=event.author.username);
    ReplyID = None;
    resuleMessage = ""
    
    #====================================================================================================

    # 文本格式
    # [Bot Molu (Bottame)]:
    # Hi!

    #====================================================================================================

    # 查看回复
    if event.reply:
        discordMessage = await DB.find_by_discord_message_id(str(event.reply.id))
        messageList = await DiscordModule.GetTables(str(event.reply.id))
        if discordMessage:
            ReplyID = discordMessage.onebot_message_id
        elif messageList:
            for segment in messageList:
                ReplyID = segment["id"];
                if segment["type"] == "file":
                    break;
        else:
            logger.warning("[Handlers_QQ] 获取Reply失败， 属性: ", str(event.reply))

    if ReplyID:
        resuleMessage += OneBotMessageSegment.reply(ReplyID)

    # 消息内容
    if event.content:
        resuleMessage += await formatAT(formatImg(event.content))

    # 贴纸
    if event.sticker_items:
        for sticker in event.sticker_items:
            if sticker.format_type.value in [1, 4]:                         # PNG/GIF
                extension = 'png' if sticker.format_type.value == 1 else 'gif'
                resuleMessage += OneBotMessageSegment.image(f"https://media.discordapp.net/stickers/{sticker.id}.{extension}")
            elif sticker.format_type.value == 2:                            # APNG
                gifPath = await apngToGif(f"https://media.discordapp.net/stickers/{sticker.id}.png")
                if not gifPath:
                    continue;
                resuleMessage += OneBotMessageSegment.image(gifPath)
            elif sticker.format_type.value == 3:                            # Lottie
                lottiePath = await lottieToGif(f"https://discord.com/stickers/{sticker.id}.json")
                if not lottiePath:
                    continue;
                resuleMessage += OneBotMessageSegment.image(lottiePath);

    # Discord嵌入式图片
    if event.embeds:
        for embed in event.embeds:
            embedType = embed.type;
            if embedType == "gifv" and embed.video is not None:
                gifFile = await QQFunc.getGIFFile(embed.video.url);
                resuleMessage += OneBotMessageSegment.image(gifFile)
            elif embedType == "image" and embed.thumbnail is not None:
                resuleMessage += OneBotMessageSegment.image(embed.thumbnail.url)

    # 附件
    if event.attachments:
        for fileInfo in event.attachments:
            # 图片
            if fileInfo.content_type and "image" in fileInfo.content_type.lower():
                resuleMessage += OneBotMessageSegment.image(fileInfo.url);
                continue;
            # 文件
            send_result = await QQFunc.sendFile(fileInfo);
            if send_result:
                message_id = next((result['message_id'] for result in send_result if 'message_id' in result), None)
                if message_id:
                    logger.info(f"File sent with message_id: {message_id}")
                    await DiscordModule.Update(str(event.id), message_id, "file")

    # 发送消息
    if resuleMessage:
        result = await QQFunc.sendGroup(OneBotMessage(resuleMessage));
        await DiscordModule.Update(str(event.id), result.get("message_id"), "content")
    

#@todo: delete_msg有异常，怀疑是Lagrange.Onebot的问题
@noticeEvent.handle()
async def handle_discord_delete_message(bot: DiscordBot, event: DiscordMessageDeleteEvent):
    if not bot_manager.OneBotObj or not isinstance(event, DiscordMessageDeleteEvent) or event.channel_id != plugin_config.discord_channel:
        return;

    # Discord撤回QQ用户消息
    discordMessage = await DB.find_by_discord_message_id(str(event.id))
    try:
        if discordMessage:
            await bot_manager.OneBotObj.delete_msg(message_id=int(discordMessage.onebot_message_id));
            return;
    except Exception as e:
        pass;
    
    # Discord用户自主撤回
    messageList = await DiscordModule.GetIDs(str(event.id))
    if not messageList or len(messageList) <= 0:
        return;

    for segment in messageList:
        await bot_manager.OneBotObj.delete_msg(message_id=int(segment));


import re
from typing import List, Optional
from nonebot.log import logger
from nonebot_dcqq_relay_plugin.config import plugin_config
from nonebot_dcqq_relay_plugin.Core.constants import bot_manager, ENCODED_FACE_PATTERN
from nonebot_dcqq_relay_plugin.Core.global_functions import getFile, getHttpxFile, generateRandomString
from nonebot.adapters.discord.api import File, MessageGet, MessageReference
from nonebot.adapters.onebot.v11 import (
    Bot as OneBotBot,
    MessageSegment as OneBotMessageSegment
)

#=================================================

async def get_user_info(bot: OneBotBot, group_id: int, user_id: int) -> tuple[str, str]:
    user_info = await bot.get_group_member_info(group_id=group_id, user_id=user_id, no_cache=True)
    user_name = f"{user_info['card'] or user_info['nickname']} (QQ: {user_id})"
    avatar_url = f"http://q.qlogo.cn/headimg_dl?dst_uin={user_id}&spec=640"
    return user_name, avatar_url

def remove_encoded_faces(text: str) -> str:
    '''移除编码的表情文本'''
    return ENCODED_FACE_PATTERN.sub('', text)

# 解析cq码
def extract_cq(message_type, message_str):
    pattern = r'\[CQ:(\w+),id=(-?\d+).*?\]'
    matches = re.findall(pattern, message_str)

    for match in matches:
        cq_type, cq_id = match
        if cq_type == message_type:
            return cq_id
    return None

#=================================================

class Discord:

    @classmethod
    async def deleteMessage(cls, MessageID: int):
        if not MessageID:
            return;
        
        findDiscordMessage = await bot_manager.DiscordBotObj.get_channel_message(
            channel_id=int(plugin_config.discord_channel),
            message_id=MessageID
        );

        if not findDiscordMessage:
            return;
        
        await bot_manager.DiscordBotObj.delete_message(
            channel_id=int(plugin_config.discord_channel),
            message_id=MessageID
        )

    @classmethod
    async def deleteWebhookMessage(cls, MessageID: int):

        if not MessageID:
            return;
        
        findDiscordMessage = await bot_manager.DiscordBotObj.get_webhook_message(
            webhook_id=bot_manager.webhook_id,
            token=bot_manager.webhook.token,
            message_id=MessageID
        )

        if not findDiscordMessage:
            return;
        
        await bot_manager.DiscordBotObj.delete_webhook_message(
            webhook_id=bot_manager.webhook_id,
            token=bot_manager.webhook.token,
            message_id=MessageID
        )

    def __init__(self, username: str, avatar_url:str):
        self.username = username;
        self.avatar_url = avatar_url;

    @staticmethod
    async def send(message: str) -> MessageGet:
        """Discord -> 发送纯文本(非webhook)"""
        return await bot_manager.DiscordBotObj.send_to(channel_id=int(plugin_config.discord_channel), message=message)

    async def reply(self, message_id: int) -> MessageGet:
        """Discord -> 提醒用户被回复了"""
        return await bot_manager.DiscordBotObj.create_message(
            channel_id=int(plugin_config.discord_channel), 
            message_reference=MessageReference(
                message_id=message_id,
                channel_id=int(plugin_config.discord_channel)
            ), 
            content=f"{self.username} replied to this message!" 
        )

    async def _execute_webhook(self, **kwargs) -> MessageGet:
        """执行webhook的通用方法"""
        return await bot_manager.DiscordBotObj.execute_webhook(
            wait=True,
            webhook_id=bot_manager.webhook.id,
            token=bot_manager.webhook.token,
            username=self.username or "Unknown User",
            avatar_url=self.avatar_url,
            **kwargs
        )

    async def sendMessage(self, message: str) -> Optional[MessageGet]:
        """Discord -> 发送纯文本消息"""
        cleaned_text = remove_encoded_faces(str(message))
        if cleaned_text.strip():  # 只有在清理后的文本不为空时才发送
            return await self._execute_webhook(content=cleaned_text)
        return None

    async def sendFile(self, file: File) -> Optional[MessageGet]:
        """Discord -> 发送单文件"""
        return await self._execute_webhook(files=[file])

    async def sendFiles(self, files: List[File]) -> Optional[MessageGet]:
        """Discord -> 发送多文件"""
        return await self._execute_webhook(files=files)

    async def sendMessageWithFiles(self, message: str, files: List[File]) -> Optional[MessageGet]:
        """Discord -> 发送消息和文件"""
        return await self._execute_webhook(content=message, files=files)

    async def sendFace(self, segment: OneBotMessageSegment) -> Optional[MessageGet]:
        """Discord -> 解析QQ表情后发送"""
        if segment.type == "image":

            # 有待优化
            file_byte, file_status_code, file_type = await getHttpxFile(segment.data['url'])
            if file_status_code == 200 and "image" in file_type:
                imgtype = ""

                # 不知道这些类型够不够用
                if "gif" in file_type:
                    imgtype = ".gif"
                elif "jpeg" in file_type:
                    imgtype = ".jpg"
                elif "png" in file_type:
                    imgtype = ".png"
                else:
                    logger.error(f"Unknown Image Type [Type: {file_type}]")
                    return None;

                # 在想是传字节码好还是文件好
                file = File(filename=generateRandomString() + imgtype, content=file_byte)
                return await self.sendFile(file)
        elif segment.type == "mface":
            return await self.sendMessage(segment.data['url'])
        elif segment.type == "face":
            emojiURL = f"https://robonyantame.github.io/QQEmojiFiles/Image/{segment.data.get('id')}.gif"
            file_byte, file_status_code = await getFile(emojiURL)
            if file_status_code == 200:
                return await self.sendMessage(emojiURL)
        return None


import re
from typing import Any, List, Optional, Union
from pathlib import Path
from nonebot.log import logger
from moviepy.editor import VideoFileClip
from nonebot_dcqq_relay_plugin.config import plugin_config;
from nonebot_dcqq_relay_plugin.Core.constants import bot_manager, EMOJI_PATTERN, DISCORD_AT_PATTERN
from nonebot_dcqq_relay_plugin.Core.global_functions import getFile, getFile_saveLocal, getFile_saveLocal2
from nonebot.adapters.onebot.v11 import Message as OneBotMessage, MessageSegment as OneBotMessageSegment
from nonebot.adapters.discord.api import Attachment as DiscordAttachment

#=================================================

# Emoji正则表达
def formatImg(content: str):
    # 如果文本空的就返回空文本
    if not content:
        return "";

    # 如果没有符合正则表达式的直接返回文本
    emojis = EMOJI_PATTERN.findall(content)
    if not emojis:
        return content;

    # 局部变量
    segments = []
    last_end = 0

    # 遍历
    for match in EMOJI_PATTERN.finditer(content):
        emoji_full = match.group(0)
        emoji_name = match.group(1)
        emoji_id = match.group(2)
        
        start = match.start()
        end = match.end()

        # 添加表情前的文本
        if start > last_end:
            segments.append(OneBotMessageSegment.text(content[last_end:start]))

        # 判断表情类型并获取相应的 URL
        if emoji_full.startswith('<a:'):
            emoji_url = f'https://cdn.discordapp.com/emojis/{emoji_id}.gif'
        else:
            emoji_url = f'https://cdn.discordapp.com/emojis/{emoji_id}.png'

        # 添加转换后的表情（使用 CQ 码）
        segments.append(OneBotMessageSegment.image(emoji_url))

        last_end = end

    # 添加最后一个表情后的文本
    if last_end < len(content):
        segments.append(OneBotMessageSegment.text(content[last_end:]))

    # 包装成OneBot消息后返回
    return OneBotMessage(segments);

def formatName(userName: str, userNick: Optional[str], global_name: Optional[str]):
    if userNick:
        return f"{userNick} ({userName})"
    elif global_name:
        return f"{global_name} ({userName})"
    else:
        return userName;

async def formatAT(content: str):
    if not content:
        return ""
    
    content = str(content)
    matches = list(DISCORD_AT_PATTERN.finditer(content))
    
    if not matches:
        return content

    segments = []
    last_end = 0

    for match in matches:
        user_id = match.group(1)
        start = match.start()
        end = match.end()
        user = await bot_manager.DiscordBotObj.get_guild_member(guild_id=int(plugin_config.discord_guild), user_id=int(user_id))

        segments.append(content[last_end:start])
        segments.append(f"@{formatName(user.nick, user.user.username, user.user.global_name)}")

        last_end = end

    segments.append(content[last_end:])
    formatted_content = ''.join(segments)

    return formatted_content

class QQ():
    
    # 构造函数
    def __init__(self, userName: str, globalName: Optional[str], userNick: Optional[str] = None):
        self.Name = formatName(userName, userNick, globalName);

    # 发送文字
    async def sendGroup(self, Message: Union[str, OneBotMessage]) -> dict[str, Any]:
        message = f"[{self.Name}]:\n{Message}";
        return await bot_manager.OneBotObj.send_group_msg(group_id=int(plugin_config.onebot_channel), message=message);
    
    # 发送图片
    async def sendImage(self, image_source: Union[str, DiscordAttachment]) -> dict[str, Any]:
        image_url = image_source if isinstance(image_source, str) else image_source.url
        image_segment = OneBotMessageSegment.image(image_url)
        return await self.sendGroup(OneBotMessage(image_segment))

    # 获得gif文件
    async def getGIFFile(self, embedURL: str) -> Optional[bytes]:    
        try:  
            FilePath, FileName = await getFile_saveLocal(embedURL, "mp4")
            if FilePath is None or FileName is None:
                return None;

            video = VideoFileClip(str(FilePath.resolve()))
            
            # 设置路径
            output_path = bot_manager.DOWNLOAD_PATH / (FileName + ".gif");
            
            # 将视频转换为 GIF
            video.write_gif(str(output_path.resolve()))
            
            # 关闭视频对象
            video.close()

            # 获取GIF字节
            saveGIFBytes = output_path.read_bytes();

            # 刪除文件
            FilePath.unlink()
            output_path.unlink()

            # 返回字节
            return saveGIFBytes

        except Exception as e:
            logger.error(f"Error in getGIFFile: {e}")
            return None

    # 发送文件
    async def sendFile(self, fileInfo: DiscordAttachment) -> Optional[List[dict[str, Any]]]:
        # Debug日志
        logger.debug(f"Download {fileInfo.filename}...");
        
        # 获取字节码
        FilePath = await getFile_saveLocal2(fileInfo.url, fileInfo.filename)
        if FilePath is None:
            logger.error("[sendFile] Failed to download file");
            return;

        results: List[dict[str, Any]] = [];

        try:
            # 当上传文件时提示是谁发送的内容
            send_result = await self.sendGroup(f"上传了文件 ({fileInfo.filename})");
            if isinstance(send_result, dict):
                results.append(send_result);
            # 上传文件
            upload_result = await bot_manager.OneBotObj.upload_group_file(
                group_id=int(plugin_config.onebot_channel), 
                file=str(FilePath.resolve()), 
                name=fileInfo.filename
            );
            if isinstance(upload_result, dict):
                results.append(upload_result);

        finally:
            # 删除文件
            FilePath.unlink(missing_ok=True)

        return results



from nonebot.log import logger

from nonebot_dcqq_relay_plugin.config import plugin_config
from nonebot_dcqq_relay_plugin.Database import DB, QQModule
from nonebot_dcqq_relay_plugin.Core.constants import messageEvent, bot_manager, noticeEvent
from nonebot_dcqq_relay_plugin.Adapters.Discord import Discord, get_user_info, extract_cq
from nonebot_dcqq_relay_plugin.Core.global_functions import getFile

from nonebot.adapters.discord.api import File, MessageGet
from nonebot.adapters.onebot.v11 import (
    Bot as OneBotBot,
    GroupMessageEvent as OneBotGroupMessageEvent,
    #MessageSegment as OneBotMessageSegment,
    GroupRecallNoticeEvent as OneBotGroupRecallNoticeEvent,
    GroupUploadNoticeEvent as OneBotGroupUploadNoticeEvent
)

#====================================================================================================

# 消息事件
@messageEvent.handle()
async def handle_qq_message(bot: OneBotBot, event: OneBotGroupMessageEvent):
    if not bot_manager.DiscordBotObj or not isinstance(event, OneBotGroupMessageEvent) or event.group_id != plugin_config.onebot_channel:
        return
    
    await QQModule.Create(str(event.message_id));

    # 防止机器人自己转发自己的消息
    login_info = await bot.get_login_info()
    if event.user_id == login_info["user_id"]:
        return;

    #====================================================================================================

    # Webhook格式
    # 用户名：Robonyan_tame (QQ: 123456789):
    # 文本：Hi!

    #====================================================================================================
    user_name, avatar_url = await get_user_info(bot, event.group_id, event.user_id);
    DiscordFunc = Discord(user_name, avatar_url);
    resultMessage = ""
    #====================================================================================================

    # 回复
    reply_id = extract_cq("reply", event.raw_message)
    if reply_id:
        onebotReplyMessageID = await DB.find_by_onebot_message_id(reply_id);
        if onebotReplyMessageID:
            res = await DiscordFunc.reply(onebotReplyMessageID.discord_message_id);
            await QQModule.Update(str(event.message_id), res.id, "reply")
        else:
            QQDB = await QQModule.GetTables(reply_id)
            if QQDB is not None:
                segment = QQDB[-1]['id']
                logger.info(f"reply_id: {segment}")
                res = await DiscordFunc.reply(int(segment));
                await QQModule.Update(str(event.message_id), str(res.id), "reply")

    # 回复
    # None

    # 遍历信息获取常见CQ码
    for segment in event.message:
        logger.debug("segment.type: " + str(segment.type));
        if segment.type == "text" and not bool(segment.data.get('text')):           # 预防特殊事件导致脚本发生错误
            logger.warning(f"遇到无法处理的文本, 进行跳过处理 [内容: {repr(segment.data)}]");
            continue;
        elif segment.type in ["image", "mface", "face"]:                            # 表情
            res = await DiscordFunc.sendFace(segment)
            if res is not None:
                await QQModule.Update(str(event.message_id), res.id, "image")
        elif segment.type == "at":                                                  # 提及他人
            atuser_name, atavatar_url  = await get_user_info(bot, event.group_id, segment.data.get("qq"));
            resultMessage += f"`@{atuser_name}` ";
        elif segment.type != "text":                                                # 孩子也不知道有啥要做的
            logger.debug(f"检测到 CQ 码：\n类型: {segment.type}\n数据: {segment.data}")
        else:                                                                       # 统统转文字处理
            resultMessage += str(segment);

    # 发送消息
    if resultMessage and resultMessage != "":
        res = await DiscordFunc.sendMessage(resultMessage)
        await QQModule.Update(str(event.message_id), res.id, "content")

# 上传群文件事件
@noticeEvent.handle()
async def handle_group_upload(bot: OneBotBot, event: OneBotGroupUploadNoticeEvent):
    # 确保事件类型是群文件上传
    if not bot_manager.DiscordBotObj or not isinstance(event, OneBotGroupUploadNoticeEvent) or event.group_id != plugin_config.onebot_channel:
        return;
    # 防止机器人自己转发自己的消息
    login_info = await bot.get_login_info()
    if event.user_id == login_info["user_id"]:
        return;

    #====================================================================================================

    await QQModule.Create(str(event.message_id));
    user_name, avatar_url = await get_user_info(bot, event.group_id, event.user_id)
    DiscordFunc = Discord(user_name, avatar_url)
    
    file_info = await bot.get_group_file_url(group_id=event.group_id, file_id=event.file.id, busid=event.file.busid)
    file_url = file_info['url']

    #====================================================================================================

    FileBytes, FileStateCode = await getFile(file_url)
    if (FileBytes == None):
        error_message = (
            f"用户 {user_name} 上传了文件：\n"
            f"但是下载文件失败，请联系管理员重新下载文件，HTTP 状态码：{FileStateCode}"
        )
        res = await DiscordFunc.send(error_message)
        await QQModule.Update(str(event.message_id), res.id, "file")
        return;

    # File类格式化
    file = File(filename=event.file.name, content=FileBytes)
    res = await DiscordFunc.sendFile(user_name, avatar_url, file);
    await QQModule.Update(str(event.message_id), res.id, "file")

# 撤回事件
@noticeEvent.handle()
async def handle_group_recall(bot: OneBotBot, event: OneBotGroupRecallNoticeEvent):
    if not bot_manager.DiscordBotObj or not isinstance(event, OneBotGroupRecallNoticeEvent) or event.group_id != plugin_config.onebot_channel:
        return;

    # 撤回discord用户消息
    onebotMessageID = await DB.find_by_onebot_message_id(event.message_id);
    if onebotMessageID:
        try:
            await Discord.deleteMessage(onebotMessageID.discord_message_id);
            return;
        except Exception as e:
            pass;

    # QQ用户自主撤回
    messageList = await QQModule.GetTables(event.message_id)
    if not messageList or len(messageList) <= 0:
        return;
    
    for segment in messageList:
        if segment["type"] != "reply":
            await Discord.deleteWebhookMessage(segment["id"]);
            continue;
        await Discord.deleteMessage(segment["id"]);
        
    


from tortoise.models import Model
from tortoise.fields.data import JSONField, DatetimeField, IntField, CharField

class OnebotMessageIndex(Model):
    id = IntField(pk=True)
    onebot_message_id = CharField(max_length=64, index=True)
    message_mapping_id = IntField()

    class Meta:
        table = "onebot_message_index"

class DiscordMessageIndex(Model):
    id = IntField(pk=True)
    discord_message_id = CharField(max_length=64, index=True)
    message_mapping_id = IntField()

    class Meta:
        table = "discord_message_index"

class MessageMapping(Model):
    id = IntField(pk=True)
    onebot_message_id = CharField(max_length=64, null=True, index=True)
    discord_message_id = CharField(max_length=64, null=True, index=True)
    onebot_message_ids = JSONField(null=True)
    discord_message_ids = JSONField(null=True)
    created_at = DatetimeField(auto_now_add=True)

    class Meta:
        table = "message_mappings"

    async def save(self, *args, **kwargs):
        await super().save(*args, **kwargs)
        
        if self.onebot_message_ids:
            for onebot_id in self.onebot_message_ids:
                await OnebotMessageIndex.get_or_create(
                    onebot_message_id=onebot_id['id'],
                    message_mapping_id=self.id
                )
        
        if self.discord_message_ids:
            for discord_id in self.discord_message_ids:
                await DiscordMessageIndex.get_or_create(
                    discord_message_id=discord_id['id'],
                    message_mapping_id=self.id
                )


import httpx, shutil, random, string, aiohttp, imageio, pyrlottie
from typing import Union, Tuple, Optional
from pathlib import Path
from nonebot.log import logger
from nonebot_dcqq_relay_plugin.Core.constants import bot_manager
from PIL import Image

def getPathFolder(path: Union[str, Path]) -> Path:
    """
    确保指定的路径存在，如果不存在则创建它。
    """
    main_path = Path(path) if isinstance(path, str) else path
    if not main_path.exists():
        main_path.mkdir(parents=True, exist_ok=True);
    return main_path

def generateRandomString(min: int = 6, max: int = 20) -> str:
    """随机生成一个最小和最大的字符串"""
    length = random.randint(min, max)  # 随机生成长度在6到20之间
    characters = string.ascii_letters + string.digits  # 包含大小写字母和数字
    random_string = ''.join(random.choice(characters) for _ in range(length))
    return random_string

def cleanDownloadFolder(path: Path):
    """
    清理下载文件夹以保证不会给缓存文件暂满
    """
    # 确保下载路径存在
    if not path.exists():
        logger.warning(f"Download folder does not exist: {str(path.resolve())}")
        return

    # 遍历并删除文件夹中的所有内容
    for item in path.iterdir():
        try:
            if item.is_file() or item.is_symlink():
                item.unlink()
                logger.debug(f"Deleted file: {item}")
            elif item.is_dir():
                shutil.rmtree(item)
                logger.debug(f"Deleted directory: {item}")
        except Exception as e:
            logger.error(f"Failed to delete {item}. Reason: {e}")

async def getFile(weblink: str) -> Tuple[Optional[bytes], int]:
    """
    异步获取指定URL的文件内容。
    """
    try:
        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=False)) as session:
            async with session.get(weblink) as response:
                if response.status == 200:
                    return await response.read(), response.status
                else:
                    logger.warning(f"Failed to fetch file. Status: {response.status}, URL: {weblink}")
                    return None, response.status
    except aiohttp.ClientError as e:
        logger.error(f"Client error when fetching file: {e}", exc_info=True)
        return None, 0
    except Exception as e:
        logger.error(f"Unexpected error when fetching file: {e}", exc_info=True)
        return None, 0
    
async def getHttpxFile(weblink: str) -> Tuple[Optional[bytes], int, Optional[str]]:
    """
    异步获取指定URL的文件内容，并确定文件类型。
    原因: https://github.com/LagrangeDev/Lagrange.Core/issues/315
    """
    try:
        async with httpx.AsyncClient(verify=False) as client:
            response = await client.get(weblink)
            if response.status_code == 200:
                data = response.content
                content_type = response.headers.get('Content-Type', None)
                return data, response.status_code, str(content_type)
            else:
                logger.warning(f"Failed to fetch file. Status: {response.status_code}, URL: {weblink}")
                return None, response.status_code, None
    except httpx.HTTPError as e:
        logger.error(f"Client error when fetching file: {e}", exc_info=True)
        return None, 0, None
    except Exception as e:
        logger.error(f"Unexpected error when fetching file: {e}", exc_info=True)
        return None, 0, None

async def getFile_saveLocal2(weblink: str, fileName: str) -> Tuple[Optional[Path]]:
    """
    获得下载文件的路径
    需要填写网站路径和完整文件名(包括文件后缀)
    **需要一个可辨认的函数名**
    """
    if not weblink:
        logger.error("[getFile_saveLocal2] - Empty Download Link")
        return None

    fileBytes, fileStateCode = await getFile(weblink)
    if not fileBytes:
        logger.error("[getFile_saveLocal2] - Empty Bytes")
        return None

    if not fileName:
        logger.error("[getFile_saveLocal2] - Empty fileName")
    
    file_path = bot_manager.DOWNLOAD_PATH / fileName;

    try:
        file_path.write_bytes(fileBytes)
        return file_path
    except Exception as e:
        logger.error(f"[getFile_saveLocal2] - file_path.write_bytes: {e}")
        return None

async def getFile_saveLocal(weblink: str, fileType: str, fileName: str = generateRandomString()) -> Tuple[Optional[Path], Optional[str]]:
    """
    获得下载文件的路径
    需要填写网站路径和文件类型
    如果fileName不填写，那么就会随机文件名称
    """
    if not weblink:
        logger.error("[getFile_saveLocal2] - Empty Download Link")
        return None

    fileBytes, fileStateCode = await getFile(weblink)
    if not fileBytes:
        logger.error("[getFile_saveLocal] - Empty Bytes")
        return None, None;

    if not fileType:
        logger.error("[getFile_saveLocal] - Empty fileType")
        return None, None;
    
    file_path = bot_manager.DOWNLOAD_PATH / (fileName + f".{fileType}");

    try:
        file_path.write_bytes(fileBytes)
        return file_path, fileName
    except Exception as e:
        logger.error(f"[getFile_saveLocal] - file_path.write_bytes: {e}")
        return None, None

async def apngToGif(apngLink: str) -> Optional[bytes]:

    if not apngLink:
        logger.error("[apngToGif] - Empty apngLink")
        return None

    apng_file, fileName = await getFile_saveLocal(apngLink, "png")
    if not apng_file:
        logger.error("[apngToGif] - Error filePath")
        return None

    gif_file = bot_manager.DOWNLOAD_PATH / (fileName + ".gif");

    try:
        reader = imageio.get_reader(apng_file, format='APNG')
        writer = imageio.get_writer(gif_file, format='GIF', mode='I')
        for frame in reader:
            writer.append_data(frame)
        writer.close()
    except Exception as e:
        logger.error(f"[apngToGif] - {e}")
        return None

    gif_bytes = gif_file.read_bytes();
    apng_file.unlink()
    gif_file.unlink()

    return gif_bytes

async def lottieToGif(lottieLink: str) -> Optional[bytes]:

    if not lottieLink:
        logger.error("[apngToGif] - Empty apngLink")
        return None
    
    lottie_file, fileName = await getFile_saveLocal(lottieLink, "json")
    if not lottie_file:
        logger.error("[lottieToGif] - Error filePath")
        return None
    
    gif_file = bot_manager.DOWNLOAD_PATH / (fileName + ".gif");

    try:
        await pyrlottie.convSingleLottie(
            lottieFile=pyrlottie.LottieFile(str(lottie_file.resolve())),
            destFiles={str(gif_file.resolve())}
        )
    except Exception as e:
        logger.error(f"[lottieToGif] - {e}")
        return None
    
    gif_bytes = gif_file.read_bytes();
    lottie_file.unlink()
    gif_file.unlink()

    return gif_bytes


from nonebot import get_plugin_config
from typing import List, Optional
from pydantic import BaseModel

class Config(BaseModel):
  data_dir: Optional[str] = None
  discord_guild: int;
  discord_channel: int;
  onebot_channel: int;

  class Config:
      extra = "ignore";

plugin_config = get_plugin_config(Config)


#from pathlib import Path
from nonebot import get_driver
from nonebot.log import logger

from nonebot.adapters.discord import Bot as DiscordBot
from nonebot.adapters.onebot.v11 import Bot as OneBotBot

from nonebot_dcqq_relay_plugin.config import plugin_config;
from nonebot_dcqq_relay_plugin.Database import DB
from nonebot_dcqq_relay_plugin.Core.constants import bot_manager, BOT_NAME
from nonebot_dcqq_relay_plugin.Core.global_functions import cleanDownloadFolder, getPathFolder

#====================================================================================================

# 以下大部分代码是对着此项目复制的: https://github.com/Autuamn/nonebot-plugin-dcqg-relay/blob/main/nonebot_plugin_dcqg_relay/__init__.py
# 谢谢大神！
driver = get_driver()

@driver.on_startup
async def init():
    # 创建路径
    pathStr = plugin_config.data_dir + "/data/" if plugin_config.data_dir is not None else "./data/";
    logger.debug(f"Plugin Data Path: {pathStr}");
    MainPath = getPathFolder(pathStr);
    bot_manager.DOWNLOAD_PATH = getPathFolder(MainPath / "download");
    bot_manager.DATABASE_PATH = getPathFolder(MainPath / "db");
    bot_manager.TEMP_PATH = getPathFolder(MainPath / "temp");
    # 初始化数据库
    await DB.init(bot_manager.DATABASE_PATH);

@driver.on_shutdown
async def clean_up():
    logger.info("机器人正在关闭...");
    # 关闭数据库
    await DB.close();
    # 清理下载文件夹
    cleanDownloadFolder(bot_manager.DOWNLOAD_PATH);

@driver.on_bot_connect
async def getDiscordBot(bot: DiscordBot):
    bot_manager.DiscordBotObj = bot;

@driver.on_bot_connect
async def getQQBot(bot: OneBotBot):
    bot_manager.OneBotObj = bot;

@driver.on_bot_connect
async def getWebhook(bot: DiscordBot):
    if not bot:
        return;

    webhooks = await bot.get_channel_webhooks(channel_id=int(plugin_config.discord_channel));
    webhookTemp = next((w for w in webhooks if w.name == BOT_NAME and w.user.username == bot.self_info.username), None);
    
    if bool(webhookTemp): 
        logger.debug("寻找到Webhook");
        bot_manager.webhook = webhookTemp;
        bot_manager.webhook_id = webhookTemp.id;
    else:
        logger.debug("没有寻找到Webhook, 正在创建");
        bot_manager.webhook = await bot.create_webhook(channel_id=int(plugin_config.discord_channel), name=BOT_NAME);
        bot_manager.webhook_id = bot_manager.webhook.id;


import setuptools

long_description = None;
requirements = None;

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="nonebot_dcqq_relay_plugin",
    version="1.1.1",
    author="Robonyantame",
    author_email="robonyantame@gmail.com",
    description="使用Nonebot2让Discord和QQ群实现互相通信",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/PawTeamClub/nonebot_dcqq_relay_plugin",
    packages=setuptools.find_packages(),
    python_requires=">=3.8, <4.0",
    classifiers=[
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 3',
    ],
    install_requires=[
        "nonebot2>=2.3.1",
        "nonebot-adapter-onebot>=2.4.3",
        "nonebot-adapter-discord>=0.1.8",
        "nonebot2[fastapi]>=2.3.1",
        "nonebot2[httpx]>=2.3.1",
        "nonebot2[websockets]>=2.3.1",
        "tortoise-orm>=0.21.4",
        "aiohttp>=3.9.5",
        "moviepy>=1.0.3",
        "imageio>=2.34.2",
        "pyrlottie>=2024.0.1",
    ],
)



import re
from pathlib import Path
from nonebot import on_message, on_notice
from nonebot.adapters.discord import Bot as DiscordBot
from nonebot.adapters.onebot.v11 import Bot as OneBotBot

#======================================================

# 初始化 NoneBot
# 给初始化用的
class BotManager:
    OneBotObj: OneBotBot = None;
    DiscordBotObj: DiscordBot = None;
    webhook_id = None;
    webhook = None;
    TEMP_PATH: Path = None;
    DOWNLOAD_PATH: Path = None;
    DATABASE_PATH: Path = None;

bot_manager = BotManager()

#======================================================

# 常量
BOT_NAME = "nonebot_dcqq_bot";

# 正则表达
EMOJI_PATTERN = re.compile(r'<a?:(\w+):(\d+)>');                #Discord Emoji
ENCODED_FACE_PATTERN = re.compile(r'&#91;([^&#]+)&#93;')        #QQ Mface Emoji
DISCORD_AT_PATTERN = re.compile(r'<@(\d+)>')

#======================================================

# 创建事件处理器
messageEvent = on_message(priority=10, block=True);
noticeEvent = on_notice(priority=5);


import json
from typing import List, Any, Optional
from .models import MessageMapping, OnebotMessageIndex, DiscordMessageIndex
from pathlib import Path
from tortoise import Tortoise
from nonebot.log import logger
from tortoise.connection import connections
from tortoise.expressions import RawSQL

#====================================================================

# Discord数据库

class DiscordModule():
    @classmethod
    async def Create(cls, discord_message_id: str):
        """创建Discord消息表"""
        return await MessageMapping.create(
            discord_message_id=discord_message_id,
            discord_message_ids=json.dumps([]),
            onebot_message_ids=json.dumps([])
        )
    @classmethod
    async def Update(cls, discord_message_id: str, onebot_message_id: str, onebot_message_type: str):
        """往Discord表添加QQ信息"""
        mapping = await MessageMapping.get(discord_message_id=discord_message_id)
        mapping.onebot_message_ids.append({"id": onebot_message_id, "type": onebot_message_type})
        await mapping.save()
    @classmethod
    async def GetTables(cls, discord_message_id: str) -> List[Any]:
        """在Discord表获取QQ消息表"""
        mapping = await MessageMapping.get_or_none(discord_message_id=discord_message_id)
        return mapping.onebot_message_ids if mapping else []
    @classmethod
    async def GetIDs(cls, discord_message_id: str) -> List[Any]:
        """获取Discord表的QQ消息ID"""
        mapping = await cls.GetTables(discord_message_id);
        return [item['id'] for item in mapping] if mapping else []
    @classmethod
    async def Del(cls, discord_message_id: str):
        await MessageMapping.filter(discord_message_id=discord_message_id).delete()

#====================================================================

# QQ数据库

class QQModule():
    @classmethod
    async def Create(cls, onebot_message_id: str):
        """创建QQ消息表"""
        return await MessageMapping.create(
            onebot_message_id=onebot_message_id,
            discord_message_ids=json.dumps([]),
            onebot_message_ids=json.dumps([])
        )
    @classmethod
    async def Update(cls, onebot_message_id: str, discord_message_id: str, discord_message_type: str):
        """往QQ消息表添加Discord信息"""
        mapping = await MessageMapping.get(onebot_message_id=onebot_message_id)
        mapping.discord_message_ids.append({"id": discord_message_id, "type": discord_message_type})
        await mapping.save()
    @classmethod
    async def GetTables(cls, onebot_message_id: str) -> List[Any]:
        """在QQ消息表获取Discord消息ID列表"""
        mapping = await MessageMapping.get_or_none(onebot_message_id=onebot_message_id)
        return mapping.discord_message_ids if mapping else []
    @classmethod
    async def GetIDs(cls, onebot_message_id: str) -> List[Any]:
        """获取QQ消息表的QQ消息ID"""
        mapping = await cls.GetTables(onebot_message_id);
        logger.debug(f"mapping len: {len(mapping)}")
        return [item['id'] for item in mapping] if mapping else []
    @classmethod
    async def GetID(cls, onebot_message_id: str, element: Any) -> Optional[Any]:
        """获取与给定元素匹配的单个Discord消息ID"""
        mapping = await cls.GetIDs(onebot_message_id);
        if not mapping:
            return None;
        for segment in mapping:
            if element == segment:
                return segment;
        return None;
    @classmethod
    async def Del(cls, onebot_message_id: str):
        await MessageMapping.filter(onebot_message_id=onebot_message_id).delete()

#====================================================================

# 主数据库方法

class DB():

    @classmethod
    async def init(cls, database_path: Path):
        """初始化数据库"""
        config = {
            "connections": {
                "nonebot_dcqq_relay_db": f"sqlite://{database_path.joinpath('data.sqlite3')}"
            },
            "apps": {
                "nonebot_dcqq_relay": {
                    #"models": ["nonebot_dcqq_relay_plugin.Database.models"],
                    "models": ["nonebot_dcqq_relay_plugin.Database.models"],
                    "default_connection": "nonebot_dcqq_relay_db",
                }
            },
        }
        await Tortoise.init(config)
        await Tortoise.generate_schemas()

    @classmethod
    async def close(cls):
        """关闭数据库"""
        await connections.close_all();

    #======================================================================

    @staticmethod
    async def find_by_onebot_message_id(onebot_message_id: str):
        # 首先查找索引表
        index = await OnebotMessageIndex.filter(onebot_message_id=onebot_message_id).first()
        if index:
            return await MessageMapping.get(id=index.message_mapping_id)

        return None;
    
    @staticmethod
    async def find_by_onebot_message_ids(onebot_message_id: str, page: int = 1, page_size: int = 100):
        offset = (page - 1) * page_size
        query = MessageMapping.filter(
            RawSQL("json_extract(onebot_message_ids, '$[*].id') LIKE ?", [f'%{onebot_message_id}%'])
        )
        total = await query.count()
        results = await query.offset(offset).limit(page_size)
        
        return {
            'total': total,
            'page': page,
            'page_size': page_size,
            'results': results
        }

    @staticmethod
    async def find_by_discord_message_id(discord_message_id: str):
        # 首先查找索引表
        index = await DiscordMessageIndex.filter(discord_message_id=discord_message_id).first()
        
        if index:
            return await MessageMapping.get(id=index.message_mapping_id)

        return None;
    
    @staticmethod
    async def find_by_discord_message_ids(discord_message_id: str, page: int = 1, page_size: int = 100):
        offset = (page - 1) * page_size
        query = MessageMapping.filter(
            RawSQL("json_extract(discord_message_ids, '$[*].id') LIKE ?", [f'%{discord_message_id}%'])
        )
        total = await query.count()
        results = await query.offset(offset).limit(page_size)
        
        return {
            'total': total,
            'page': page,
            'page_size': page_size,
            'results': results
        }